#include < amxmodx >
#include < amxmisc >
#include < cromchat >

#define PLUGIN_VERSION "1.4"

#if !defined MAX_AUTHID_LENGTH
#define MAX_AUTHID_LENGTH    35
#endif

#if !defined MAX_DATE_LENGTH
#define MAX_DATE_LENGTH      12
#endif

enum _:VisitorData
{
	Visitor_Date[ MAX_DATE_LENGTH ],
	Visitor_SteamId[ MAX_AUTHID_LENGTH ]
}

enum _:ConfigData
{
	VC_METHOD,
	VC_ADMINS_ONLY,
	VC_ADMIN_FLAG,
	VC_HUD_COLOR_R,
	VC_HUD_COLOR_G,
	VC_HUD_COLOR_B,
	Float:VC_HUD_POSITION_X,
	Float:VC_HUD_POSITION_Y,
	Float:VC_HUD_DURATION,
	VC_MOTD_MESSAGE[ 512 ],
	VC_SAVE_FILE[ 64 ]
}

enum
{
	VC_CHAT = 1,
	VC_HUD,
	VC_MOTD
}

new Array:g_aSteamIds;

new g_szConfigsDir[ 64 ];

new g_iConfig[ ConfigData ];

new const g_szConfigFile[ ] = "VisitorCounter_Configuration.ini";

public plugin_precache( )
{
	get_configsdir( g_szConfigsDir, charsmax( g_szConfigsDir ) );
	
	g_aSteamIds = ArrayCreate( MAX_AUTHID_LENGTH );
	
	ReadConfig( );
}

public plugin_init( )
{
	register_plugin( "Visitor Counter", PLUGIN_VERSION, "DoNii" );

	CC_SetPrefix( "&x04[Visitor Counter]" );
	
	register_dictionary( "visitor_counter.txt" );
	
	register_cvar( "visitor_counter_version", PLUGIN_VERSION, FCVAR_SERVER | FCVAR_SPONLY | FCVAR_UNLOGGED );
	
	register_clcmd( "say /visitors", "OnVisitorCounter" );
	register_clcmd( "say_team /visitors", "OnVisitorCounter" );
}

public client_putinserver( id )
{
	if( ! is_user_bot( id ) )
	{
		Check( id );
	}
	return PLUGIN_CONTINUE;
}

Check( id )
{
	RetrieveSteamIds( );
	
	new szAuthId[ MAX_AUTHID_LENGTH ], szDate[ MAX_DATE_LENGTH ];
	get_time( "%m/%d/%Y", szDate, charsmax( szDate ) );
	get_user_authid( id, szAuthId, charsmax( szAuthId ) );

	if( ! SteamIdExists( szAuthId, szDate ) )
	{
		SaveSteamId( szAuthId );
	}
}

bool:SteamIdExists( const szAuthId[ ], const szDate[ ] )
{
	new eData[ VisitorData ], bool:bExists;

	for( new i; i < ArraySize( g_aSteamIds ); i++ )
	{
		ArrayGetArray( g_aSteamIds, i, eData );
		
		if( equali( eData[ Visitor_SteamId ], szAuthId ) && equali( eData[ Visitor_Date ], szDate ) ) // we're allowing the same steamid if the date is different
		{
			bExists = true;
			break;
		}
	}
	return bExists;
}

SaveSteamId( const szAuthId[ ] )
{
	new szFilePath[ 128 ];
	formatex( szFilePath, charsmax( szFilePath ), "%s/%s", g_szConfigsDir, g_iConfig[ VC_SAVE_FILE ] );
	new iFile = fopen( szFilePath, "r+" );
	
	if( iFile )
	{			
		new szByteVal[ 1 ], szDate[ MAX_DATE_LENGTH ];
		get_time( "%m/%d/%Y", szDate, charsmax( szDate ) );
		
		fseek( iFile , -1 , SEEK_END );
		fread_raw( iFile , szByteVal , sizeof( szByteVal ) , BLOCK_BYTE );
		fseek( iFile , 0 , SEEK_END );
		
		fprintf( iFile, "%s%s = %s", ( szByteVal[ 0 ] == 10 ) ? "" : "^n", szDate, szAuthId );
		fclose( iFile );
	}
}

RetrieveSteamIds( )
{
	ArrayClear( g_aSteamIds );

	new szFilePath[ 128 ], szData[ 64 ];	
	formatex( szFilePath, charsmax( szFilePath ), "%s/%s", g_szConfigsDir, g_iConfig[ VC_SAVE_FILE ] );
	
	new iFile = fopen( szFilePath, "rt" );
	
	if( iFile )
	{
		while( fgets( iFile, szData, charsmax( szData ) ) )
		{
			trim( szData );
			
			switch( szData[ 0 ] )
			{
			case EOS, '#', ';', '/':
				{
					continue;
				}
				
			default:
				{
					new szKey[ MAX_DATE_LENGTH ], szValue[ MAX_AUTHID_LENGTH ], eData[ VisitorData ];
					
					strtok( szData, szKey, charsmax( szKey ), szValue, charsmax( szValue ), '=' );
					
					trim( szKey ); 
					trim( szValue );
					
					copy( eData[ Visitor_Date ], charsmax( eData[ Visitor_Date ] ), szKey );
					copy( eData[ Visitor_SteamId ], charsmax( eData[ Visitor_SteamId ] ), szValue );
					
					ArrayPushArray( g_aSteamIds, eData );
				}
			}
		}
		fclose( iFile );
	}
	return PLUGIN_CONTINUE;
}

public OnVisitorCounter( id )
{
	if( g_iConfig[ VC_ADMINS_ONLY ] )
	{
		if( ~ get_user_flags( id ) & g_iConfig[ VC_ADMIN_FLAG ] )
		{
			client_print( id, print_chat, "%L", id, "NO_ACCESS" );
			return PLUGIN_HANDLED;
		}
	}

	RetrieveSteamIds( );

	new eData[ VisitorData ], szTodaysDate[ MAX_DATE_LENGTH ], iVisitorCounter;
	get_time( "%m/%d/%Y", szTodaysDate, charsmax( szTodaysDate ) );
	
	for( new i; i < ArraySize( g_aSteamIds ); i++ )
	{
		ArrayGetArray( g_aSteamIds, i, eData );
		
		if( equali( szTodaysDate, eData[ Visitor_Date ] ) )
		{
			iVisitorCounter++;
		}
	}

	new szYesterdaysDate[ MAX_DATE_LENGTH ], bool:bRise;
	RemoveDaysFromDate( szTodaysDate, 1, szYesterdaysDate, charsmax( szYesterdaysDate ) );
	
	new iYesterdayVisits = GetYesterdaysVisits( szYesterdaysDate );
	new Float:fCalculatePercentage;
	
	if( ! iYesterdayVisits )
	{
		fCalculatePercentage = iVisitorCounter * 100.0;
		bRise = true;
	}
	
	else if( iVisitorCounter > iYesterdayVisits ) // more players than yesterday
	{
		fCalculatePercentage = Float:( iVisitorCounter - iYesterdayVisits );
		
		fCalculatePercentage /= Float:iYesterdayVisits;
		fCalculatePercentage *= 100.0;
		
		bRise = true;
	}
	
	else if( iYesterdayVisits > iVisitorCounter ) // less players than yesterday
	{
		fCalculatePercentage = Float:( iYesterdayVisits - iVisitorCounter );
		
		fCalculatePercentage /= Float:iVisitorCounter;
		fCalculatePercentage *= 100.0;
		
		bRise = false;
	}
	
	else // neutral so we'll just go with rise
	{
		bRise = true;
	}
	
	switch( g_iConfig[ VC_METHOD ] )
	{
	case VC_CHAT:
		{
			CC_SendMessage( id, "%L %L %L", id, iVisitorCounter == 1 ? "PEOPLE_VISIT_TODAY_SINGULAR" : "PEOPLE_VISIT_TODAY_PLURAL", iVisitorCounter, id, bRise ? "STATISTIC_RISE" : "STATISTIC_LOSS", fCalculatePercentage, id, iYesterdayVisits == 1 ? "YESTERDAY_VISITORS_SINGULAR" : "YESTERDAY_VISITORS_PLURAL", iYesterdayVisits );
		}

	case VC_HUD:
		{
			new szMessage[ 256 ];
			formatex( szMessage, charsmax( szMessage ), "&x00~~~ %L %L %L ~~~", id, iVisitorCounter == 1 ? "PEOPLE_VISIT_TODAY_SINGULAR" : "PEOPLE_VISIT_TODAY_PLURAL", iVisitorCounter, id, bRise ? "STATISTIC_RISE" : "STATISTIC_LOSS", fCalculatePercentage, id, iYesterdayVisits == 1 ? "YESTERDAY_VISITORS_SINGULAR" : "YESTERDAY_VISITORS_PLURAL", iYesterdayVisits );
			
			CC_RemoveColors( szMessage, charsmax( szMessage ), true );
			
			set_hudmessage( g_iConfig[ VC_HUD_COLOR_R ], g_iConfig[ VC_HUD_COLOR_G ], g_iConfig[ VC_HUD_COLOR_B ], g_iConfig[ VC_HUD_POSITION_X ], g_iConfig[ VC_HUD_POSITION_Y ], 0, 0.0, g_iConfig[ VC_HUD_DURATION ] );
			show_hudmessage( id, szMessage );
		}
		
	case VC_MOTD:
		{
			new szMessage[ 512 ];
			formatex( szMessage, charsmax( szMessage ), "&x00%s", g_iConfig[ VC_MOTD_MESSAGE ] );
			
			new szNum[ 8 ], szTranslation[ 512 ];
			num_to_str( iVisitorCounter, szNum, charsmax( szNum ) );
			
			formatex( szTranslation, charsmax( szTranslation ), "%L %L %L", id, iVisitorCounter == 1 ? "PEOPLE_VISIT_TODAY_SINGULAR" : "PEOPLE_VISIT_TODAY_PLURAL", iVisitorCounter, id, bRise ? "STATISTIC_RISE" : "STATISTIC_LOSS", fCalculatePercentage, id, iYesterdayVisits == 1 ? "YESTERDAY_VISITORS_SINGULAR" : "YESTERDAY_VISITORS_PLURAL", iYesterdayVisits );
			
			CC_RemoveColors( szTranslation, charsmax( szTranslation ), true );
			
			replace_all( szMessage, charsmax( szMessage ), "%d", szNum );
			replace_all( szMessage, charsmax( szMessage ), "%L", szTranslation );
			
			show_motd( id, szMessage, "Visitor Counter" );
		}
	}
	return PLUGIN_CONTINUE;
}

GetYesterdaysVisits( const szDate[ ] )
{
	new eData[ VisitorData ], iVisitorCounter;
	
	for( new i; i < ArraySize( g_aSteamIds ); i++ )
	{
		ArrayGetArray( g_aSteamIds, i, eData );
		
		if( equali( szDate, eData[ Visitor_Date ] ) )
		{
			iVisitorCounter++;
		}
	}
	return iVisitorCounter
}

ReadConfig( )
{
	new szFile[ 128 ], szData[ 512 ];
	formatex( szFile, charsmax( szFile ), "%s/%s", g_szConfigsDir, g_szConfigFile );
	
	new iFile = fopen( szFile, "rt" );
	
	if( iFile )
	{
		while( fgets( iFile, szData, charsmax( szData ) ) )
		{
			trim( szData );
			remove_quotes( szData );
			
			switch( szData[ 0 ] )
			{
			case EOS, '#', ';', '/':
				{
					continue;
				}

			default:
				{
					new szKey[ 32 ], szValue[ 512 ];
					strtok( szData, szKey, charsmax( szKey ), szValue, charsmax( szValue ), '=' );
					
					trim( szKey );
					trim( szValue );
					
					remove_quotes( szKey );
					remove_quotes( szValue );
					
					if( ! szValue[ 0 ] || ! szKey[ 0 ] )
					{
						continue;
					}
					
					if( equali( szKey, "VC_METHOD" ) )
					{
						g_iConfig[ VC_METHOD ] = str_to_num( szValue );
					}
					
					else if( equali( szKey, "VC_HUD_COLOR_R" ) )
					{
						g_iConfig[ VC_HUD_COLOR_R ] = str_to_num( szValue );
					}
					
					else if( equali( szKey, "VC_HUD_COLOR_G" ) )
					{
						g_iConfig[ VC_HUD_COLOR_G ] = str_to_num( szValue );
					}
					
					else if( equali( szKey, "VC_HUD_COLOR_B" ) )
					{
						g_iConfig[ VC_HUD_COLOR_B ] = str_to_num( szValue );
					}
					
					else if( equali( szKey, "VC_HUD_POSITION_X" ) )
					{
						g_iConfig[ VC_HUD_POSITION_X ] = _:str_to_float( szValue );
					}
					
					else if( equali( szKey, "VC_HUD_POSITION_Y" ) )
					{
						g_iConfig[ VC_HUD_POSITION_Y ] = _:str_to_float( szValue );
					}
					
					else if( equali( szKey, "VC_HUD_DURATION" ) )
					{
						g_iConfig[ VC_HUD_DURATION ] = _:str_to_float( szValue );
					}
					
					else if( equali( szKey, "VC_MOTD_MESSAGE" ) )
					{
						copy( g_iConfig[ VC_MOTD_MESSAGE ], charsmax( g_iConfig[ VC_MOTD_MESSAGE ] ), szValue );
					}
					
					else if( equali( szKey, "VC_ADMINS_ONLY" ) )
					{
						g_iConfig[ VC_ADMINS_ONLY ] = str_to_num( szValue );
					}
					
					else if( equali( szKey, "VC_ADMIN_FLAG" ) )
					{
						g_iConfig[ VC_ADMIN_FLAG ] = read_flags( szValue );
					}
					
					else if( equali( szKey, "VC_SAVE_FILE" ) )
					{
						copy( g_iConfig[ VC_SAVE_FILE ], charsmax( g_iConfig[ VC_SAVE_FILE ] ), szValue );
					}
				}
			}
		}
		fclose( iFile );
	}
}

RemoveDaysFromDate( const szTodaysDate[ ], const iDays, szDate[ ], const iSize )
{
	new const szFormatRule[ ] = "%m/%d/%Y";
	
	new iCurrentTimeStamp = parse_time( szTodaysDate, szFormatRule );
	iCurrentTimeStamp = iCurrentTimeStamp - ( iDays * 86400 );
	format_time( szDate, iSize, szFormatRule, iCurrentTimeStamp );
}